/*
** isight.c
** 
** Made by (Arne Caspari)
** Login   <arne@localhost>
** 
** Started on  Thu Jun  5 14:33:13 2008 Arne Caspari
*/

#include "config.h"
#include "isight.h"

#include <limits.h>
#include <stdlib.h>
#include <linux/types.h>
#include <string.h>
#include <libgen.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/videodev2.h>

#include "uvc_compat.h"

#if V4L2_DEBUG
#define DEBUG
#endif

#include "debug.h"

#define N_(x) x

#define FOURCC(a,b,c,d) (unsigned int)((((unsigned int)d)<<24)+(((unsigned int)c)<<16)+(((unsigned int)b)<<8)+a)

static int get_usb_ids( const char *path, unsigned short *vid, unsigned short *pid )
{
   char respath[PATH_MAX];
   char *basen;
   char prodidpath[PATH_MAX];
   char vendidpath[PATH_MAX];
   char devicepath[PATH_MAX];
   char tmp[PATH_MAX];
   FILE *f;

   strcpy( tmp, path );
   basen = basename( tmp );
   sprintf( devicepath, "/sys/class/video4linux/%s/device", basen );
   
   if( !realpath( devicepath, respath ) )
   {
      TRACE( "could not resolve path '%s'\n", devicepath );
      return 0;
   }
   
   strcpy( vendidpath, devicepath );
   strcat( vendidpath, "/../idVendor" );

   f = fopen( vendidpath, "r" );
   if( !f )
   {
      TRACE( "could not open '%s'\n", vendidpath );
      return 0;
   }
   else
   {
      char buf[5];
      memset( buf, 0x0, sizeof( buf ) );
      fscanf( f, "%hx", vid );
      fclose( f );
   }
   

   strcpy( prodidpath, devicepath );
   strcat( prodidpath, "/../idProduct" );
   
   f = fopen( prodidpath, "r" );
   if( !f )
   {
      TRACE( "could not open '%s'\n", prodidpath );
      return 0;
   }
   else
   {
      char buf[5];
      memset( buf, 0x0, sizeof( buf ) );
      fscanf( f, "%hx", pid );
      fclose( f );
   }
   
   return 1;
}

int isight_probe( int fd, const char *path )
{
   int ret = 0;
   unsigned short vid, pid;

   get_usb_ids( path, &vid, &pid );

   if( ( vid == 0x05ac ) && ( pid == 0x8501 ) )
   {
      TRACE( "Found Apple Built-in iSight\n" );
      ret = 1;
   }
   
   return ret;
}


int isight_count_ext_property( int fd )
{
   return 0;
}


unicap_status_t isight_enumerate_properties( int fd, int index, unicap_property_t *property )
{
   return STATUS_NO_MATCH;
}

unicap_status_t isight_override_property( int fd, struct v4l2_queryctrl *ctrl, unicap_property_t *property )
{
   unicap_status_t status = STATUS_NO_MATCH;
   
   switch( ctrl->id )
   {
      case V4L2_CID_EXPOSURE_ABSOLUTE:
/* 	 if( property ) */
/* 	 { */
/* 	    strcpy( property->identifier, N_("shutter") ); */
/* 	    strcpy( property->category, N_("exposure") ); */
/* 	    strcpy( property->unit, "s" ); */
/* 	    property->flags_mask = UNICAP_FLAGS_MANUAL | UNICAP_FLAGS_AUTO; */
/* 	    property->flags = UNICAP_FLAGS_AUTO; */
/* 	    property->type = UNICAP_PROPERTY_TYPE_RANGE; */
/* 	    property->range.min = (double)ctrl->minimum / 10000.0; */
/* 	    property->range.max = (double)ctrl->maximum / 10000.0; */
/* 	    property->value = (double)ctrl->default_value / 10000.0; */
/* 	 } */
/* 	 status = STATUS_SUCCESS; */
/* 	 break; */	 

      default:
	 break;
	 
   }
   
	    
   return status;
}

unicap_status_t isight_set_property( int fd, unicap_property_t *property )
{
   unicap_status_t status = STATUS_NO_MATCH;

   return status;
}

unicap_status_t isight_get_property( int fd, unicap_property_t *property )
{
   unicap_status_t status = STATUS_NO_MATCH;
   return status;
}

unicap_status_t isight_fmt_get( struct v4l2_fmtdesc *v4l2fmt, struct v4l2_cropcap *cropcap, char **identifier, unsigned int *fourcc, int *bpp )
{
   unicap_status_t status = STATUS_NO_MATCH;

   TRACE( "fmt_get desc='%s', defrect.width=%d\n", v4l2fmt->description, cropcap->defrect.width );

   if( v4l2fmt->pixelformat == FOURCC( 'Y', 'U', 'Y', '2' ) )
   {
      if( identifier )
      {
	 *identifier = "UYVY";
      }
      if( fourcc )
      {
	 *fourcc = FOURCC( 'U', 'Y', 'V', 'Y' );
      }
      if( bpp )
      {
	 *bpp = 16;
      }
      status = STATUS_SUCCESS;
   }
      
   TRACE( "fmt get: id = %s width= %d\n", identifier ? *identifier : "", cropcap->defrect.width );
   return status;
}


unicap_status_t isight_tov4l2format( unicap_format_t *format )
{
   unicap_status_t status = STATUS_NO_MATCH;
   
   if( format->fourcc == FOURCC( 'U', 'Y', 'V', 'Y' ) )
   {
      format->fourcc = FOURCC( 'Y', 'U', 'Y', '2' );
      format->bpp = 16;
      status = STATUS_SUCCESS;
   }
   
   return status;
}
