/*
** tiseuvccam.c
** 
** Made by (Arne Caspari)
** Login   <arne@localhost>
** 
** Started on  Thu Jun  5 14:33:13 2008 Arne Caspari
*/

#include "config.h"
#include "tiseuvccam.h"

#include <limits.h>
#include <stdlib.h>
#include <linux/types.h>
#include <string.h>
#include <libgen.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/videodev2.h>

#include "uvc_compat.h"

#if V4L2_DEBUG
#define DEBUG
#endif

#include "debug.h"

#define N_(x) x

#define FOURCC(a,b,c,d) (unsigned int)((((unsigned int)d)<<24)+(((unsigned int)c)<<16)+(((unsigned int)b)<<8)+a)

static int get_usb_ids( const char *path, unsigned short *vid, unsigned short *pid )
{
   char respath[PATH_MAX];
   char *basen;
   char prodidpath[PATH_MAX];
   char vendidpath[PATH_MAX];
   char devicepath[PATH_MAX];
   char tmp[PATH_MAX];
   FILE *f;

   strcpy( tmp, path );
   basen = basename( tmp );
   sprintf( devicepath, "/sys/class/video4linux/%s/device", basen );
   
   if( !realpath( devicepath, respath ) )
   {
      TRACE( "could not resolve path '%s'\n", devicepath );
      return 0;
   }
   
   strcpy( vendidpath, devicepath );
   strcat( vendidpath, "/../idVendor" );

   f = fopen( vendidpath, "r" );
   if( !f )
   {
      TRACE( "could not open '%s'\n", vendidpath );
      return 0;
   }
   else
   {
      char buf[5];
      memset( buf, 0x0, sizeof( buf ) );
      fscanf( f, "%hx", vid );
      fclose( f );
   }
   

   strcpy( prodidpath, devicepath );
   strcat( prodidpath, "/../idProduct" );
   
   f = fopen( prodidpath, "r" );
   if( !f )
   {
      TRACE( "could not open '%s'\n", prodidpath );
      return 0;
   }
   else
   {
      char buf[5];
      memset( buf, 0x0, sizeof( buf ) );
      fscanf( f, "%hx", pid );
      fclose( f );
   }
   
   return 1;
}

int tiseuvccam_probe( int fd, const char *path )
{
   int ret = 0;
   unsigned short vid, pid;

   get_usb_ids( path, &vid, &pid );

   if( ( vid == 0x199e ) && ( pid == 0x8201 ) )
   {
      TRACE( "Found TIS EUVCCAM\n" );
      ret = 1;
   }
   
   return ret;
}


int tiseuvccam_count_ext_property( int fd )
{
   return 0;
}


unicap_status_t tiseuvccam_enumerate_properties( int fd, int index, unicap_property_t *property )
{
   return STATUS_NO_MATCH;
}

unicap_status_t tiseuvccam_override_property( int fd, struct v4l2_queryctrl *ctrl, unicap_property_t *property )
{
   unicap_status_t status = STATUS_NO_MATCH;
   
   switch( ctrl->id )
   {
#ifdef V4L2_CID_PRIVACY_CONTROL
      case V4L2_CID_PRIVACY_CONTROL:
	 if( property )
	 {
	    strcpy( property->identifier, N_("trigger") );
	    strcpy( property->category, N_("device" ) );
	    strcpy( property->unit, "" );
	    property->flags_mask = UNICAP_FLAGS_MANUAL | UNICAP_FLAGS_ON_OFF;
	    property->flags = UNICAP_FLAGS_MANUAL;
	    property->type = UNICAP_PROPERTY_TYPE_FLAGS;
	 }
	 status = STATUS_SUCCESS;
	 
	 break;
#endif
      case V4L2_CID_EXPOSURE_AUTO:
	 status = STATUS_SKIP_CTRL;
	 break;

      case V4L2_CID_EXPOSURE_ABSOLUTE:
	 if( property )
	 {
	    strcpy( property->identifier, N_("shutter") );
	    strcpy( property->category, N_("exposure") );
	    strcpy( property->unit, "s" );
	    property->flags_mask = UNICAP_FLAGS_MANUAL | UNICAP_FLAGS_AUTO;
	    property->flags = UNICAP_FLAGS_AUTO;
	    property->type = UNICAP_PROPERTY_TYPE_RANGE;
	    property->range.min = (double)ctrl->minimum / 10000.0;
	    property->range.max = (double)ctrl->maximum / 10000.0;
	    property->value = (double)ctrl->default_value / 10000.0;
	 }
	 status = STATUS_SUCCESS;
	 break;

/* 	 case V4L2_CID_WHITE */
	 

      default:
	 break;
	 
   }
   
	    
   return status;
}

unicap_status_t tiseuvccam_set_property( int fd, unicap_property_t *property )
{
   unicap_status_t status = STATUS_NO_MATCH;

   if( !strcmp( property->identifier, "trigger" ) )
   {
#ifdef V4L2_CID_PRIVACY_CONTROL
      struct v4l2_control ctrl;
      memset( &ctrl, 0x0, sizeof( ctrl ) );
      
      ctrl.id = V4L2_CID_PRIVACY_CONTROL;
      ctrl.value = ( ( property->flags & UNICAP_FLAGS_ON_OFF ) == UNICAP_FLAGS_ON_OFF ) ? 1 : 0;
      if( ioctl( fd, VIDIOC_S_CTRL, &ctrl ) < 0 )
      {
	 TRACE( "VIDIOC_S_CTRL failed\n" );
	 return STATUS_FAILURE;
      }
      else
      {
	 status = STATUS_SUCCESS;
      }
#else
      return STATUS_FAILURE;
#endif
   } 
   else if( !strcmp( property->identifier, "shutter" ) )
   {
      struct v4l2_control ctrl;
      memset( &ctrl, 0x0, sizeof( ctrl ) );
      ctrl.id = V4L2_CID_EXPOSURE_AUTO;
      ctrl.value = ( property->flags & UNICAP_FLAGS_AUTO ) ? V4L2_EXPOSURE_AUTO : V4L2_EXPOSURE_MANUAL;
      if( ioctl( fd, VIDIOC_S_CTRL, &ctrl ) < 0 )
      {
	 TRACE( "VIDIOC_S_CTRL failed\n" );
	 return STATUS_FAILURE;
      }
	  
      if( property->flags & UNICAP_FLAGS_MANUAL )
      {
	 memset( &ctrl, 0x0, sizeof( ctrl ) );
	 ctrl.id = V4L2_CID_EXPOSURE_ABSOLUTE;
	 ctrl.value = property->value * 10000;
	 if( ioctl( fd, VIDIOC_S_CTRL, &ctrl ) < 0 )
	 {
	    TRACE( "VIDIOC_S_CTRL failed\n" );
	    return STATUS_FAILURE;
	 }

	 status = STATUS_SUCCESS;
      }
   }

   return status;
}

unicap_status_t tiseuvccam_get_property( int fd, unicap_property_t *property )
{
   unicap_status_t status = STATUS_NO_MATCH;

   if( !strcmp( property->identifier, "trigger" ) )
   {
#ifdef V4L2_CID_PRIVACY_CONTROL
      struct v4l2_control ctrl;
      memset( &ctrl, 0x0, sizeof( ctrl ) );
      
      ctrl.id = V4L2_CID_PRIVACY_CONTROL;
      if( ioctl( fd, VIDIOC_G_CTRL, &ctrl ) < 0 )
      {
	 TRACE( "VIDIOC_G_CTRL failed\n" );
	 return STATUS_FAILURE;
      }
      else
      {
	 status = STATUS_SUCCESS;
      }

      if( ctrl.value )
      {
	 property->flags = UNICAP_FLAGS_MANUAL | UNICAP_FLAGS_ON_OFF;
      }
      else
      {
	 property->flags = UNICAP_FLAGS_MANUAL;
      }      
#else
      return STATUS_FAILURE;
#endif
   }
   else if( !strcmp( property->identifier, "shutter" ) )
   {
      struct v4l2_control ctrl;
      memset( &ctrl, 0x0, sizeof( ctrl ) );      
      ctrl.id = V4L2_CID_EXPOSURE_AUTO;
      if( ioctl( fd, VIDIOC_G_CTRL, &ctrl ) < 0 )
      {
	 TRACE( "VIDIOC_G_CTRL ( V4L2_CID_EXPOSURE_AUTO ) failed\n" );
	 return STATUS_FAILURE;
      }
      else
      {
	 status = STATUS_SUCCESS;
      }
      
      if( ctrl.value == V4L2_EXPOSURE_MANUAL )
      {
	 property->flags = UNICAP_FLAGS_MANUAL;
      }
      else
      {
	 property->flags = UNICAP_FLAGS_AUTO;
      }
      
      memset( &ctrl, 0x0, sizeof( ctrl ) );
      ctrl.id = V4L2_CID_EXPOSURE_ABSOLUTE;
      if( ioctl( fd, VIDIOC_G_CTRL, &ctrl ) < 0 )
      {
	 TRACE( "VIDIOC_G_CTRL failed!\n" );
	 return STATUS_FAILURE;
      }
      else
      {
	 property->value = ctrl.value / 10000.0;
	 status = STATUS_SUCCESS;
      }
   }   
   
   return status;
}

unicap_status_t tiseuvccam_fmt_get( struct v4l2_fmtdesc *v4l2fmt, struct v4l2_cropcap *cropcap, char **identifier, unsigned int *fourcc, int *bpp )
{
   unicap_status_t status = STATUS_NO_MATCH;

   TRACE( "fmt_get desc='%s', defrect.width=%d\n", v4l2fmt->description, cropcap->defrect.width );

   if( ( v4l2fmt->pixelformat == FOURCC( 'Y', 'U', 'Y', 'V' ) ) && ( ( cropcap->defrect.width == 372 ) || ( cropcap->defrect.width == 320 ) ) )
   {
      if( identifier )
      {
	 *identifier = "Y800";
      }
      if( fourcc )
      {
	 *fourcc = FOURCC( 'Y', '8', '0', '0' );
      }
      if( bpp )
      {
	 *bpp = 8;
      }

      cropcap->defrect.width *= 2;
      cropcap->bounds.width *= 2;
      
      status = STATUS_SUCCESS;
   }
   else if( ( v4l2fmt->pixelformat == FOURCC( 'U', 'Y', 'V', 'Y' ) ) && ( ( cropcap->defrect.width == 372 ) || ( cropcap->defrect.width == 320 ) ) )
   {
      TRACE( "skip format: %s\n", v4l2fmt->description );
      status = STATUS_SKIP_CTRL;
   }
      
   TRACE( "fmt get: id = %s width= %d\n", identifier ? *identifier : "", cropcap->defrect.width );
   return status;
}

#ifdef VIDIOC_ENUM_FRAMESIZES
unicap_status_t tiseuvccam_override_framesize( struct v4l2_frmsizeenum *frms )
{
   unicap_status_t status = STATUS_NO_MATCH;
   
   if( frms->discrete.height == 480 && 
       ( ( frms->discrete.width == 372 ) || ( frms->discrete.width == 320 ) ) )
   {
      frms->discrete.width *= 2;
      status = STATUS_SUCCESS;
   }
   
   return status;
}
#endif

unicap_status_t tiseuvccam_tov4l2format( unicap_format_t *format )
{
   unicap_status_t status = STATUS_NO_MATCH;
   
   if( format->fourcc == FOURCC( 'Y', '8', '0', '0' ) )
   {
      format->fourcc = FOURCC( 'Y', 'U', 'Y', 'V' );
      format->size.width /= 2;
      format->bpp = 16;
      status = STATUS_SUCCESS;
   }
   
   return status;
}
